﻿using System;
using RimWorld;
using HarmonyLib;
using UnityEngine;
using Verse;


namespace NewRatkin
{
    [DefOf]
    public static class ShotGunDefOf
    {
        public static ThingDef GunlanceExplosion;
        public static ThingDef GunlancePreIgnition;
        public static ThingDef GunlanceAfterIgnition;
    }

    public class Verb_SpreadShot:Verb_Shoot
    {
        public override void WarmupComplete()
        {
            base.WarmupComplete();
        }
        public override bool CanHitTarget(LocalTargetInfo targ)
        {
            return base.CanHitTarget(targ);
        }
        public override ITargetingSource DestinationSelector => base.DestinationSelector;

        public override bool Available()
        {
            return base.Available();
        }
        public override bool CanHitTargetFrom(IntVec3 root, LocalTargetInfo targ)
        {
            return base.CanHitTargetFrom(root, targ);
        }

        protected override float EffectiveRange => base.EffectiveRange;

        public override Verb GetVerb => base.GetVerb;

        public override void ExposeData()
        {
            base.ExposeData();
        }

        protected override bool TryCastShot()
		{
			if (this.currentTarget.HasThing && this.currentTarget.Thing.Map != this.caster.Map)
			{
				return false;
			}
			ThingDef projectileDef = this.Projectile;
			if (projectileDef == null)
			{
				return false;
			}
			ShootLine shootLine;
			bool canShoot = base.TryFindShootLineFromTo(this.caster.Position, this.currentTarget, out shootLine);
			if (this.verbProps.stopBurstWithoutLos && !canShoot)
			{
				return false;
			}
			Thing launcher = this.caster;
			Thing equipment = base.EquipmentSource;
			Vector3 drawPos = this.caster.DrawPos;
			Projectile projectileThing = (Projectile)GenSpawn.Spawn(projectileDef, shootLine.Source, this.caster.Map, WipeMode.Vanish);
			ShotReport shotReport = ShotReport.HitReportFor(this.caster, this, this.currentTarget);
			Thing randomCoverToMissInto = shotReport.GetRandomCoverToMissInto();
			ThingDef targetCoverDef = (randomCoverToMissInto != null) ? randomCoverToMissInto.def : null;
			ProjectileHitFlags projectileHitFlag = ProjectileHitFlags.NonTargetWorld;

			if (!Rand.Chance(shotReport.AimOnTargetChance_IgnoringPosture))
			{
				shootLine.ChangeDestToMissWild(shotReport.AimOnTargetChance_StandardTarget);
				if (Rand.Chance(0.5f) && this.canHitNonTargetPawnsNow)
				{
					projectileHitFlag |= ProjectileHitFlags.NonTargetPawns;
				}
				ShootProjectile(projectileThing,launcher, launcher.DrawPos, shootLine.Dest, this.currentTarget, projectileHitFlag, equipment, targetCoverDef);
				return true;
			}
			if (this.currentTarget.Thing != null && this.currentTarget.Thing.def.category == ThingCategory.Pawn && !Rand.Chance(shotReport.PassCoverChance))
			{
				if (this.canHitNonTargetPawnsNow)
				{
					projectileHitFlag |= ProjectileHitFlags.NonTargetPawns;
				}
				ShootProjectile(projectileThing, launcher, launcher.DrawPos, randomCoverToMissInto, this.currentTarget, projectileHitFlag, equipment, targetCoverDef);
				return true;
			}

			projectileHitFlag = ProjectileHitFlags.IntendedTarget;
			if (this.canHitNonTargetPawnsNow)
			{
				projectileHitFlag |= ProjectileHitFlags.NonTargetPawns;
			}
			if (!this.currentTarget.HasThing || this.currentTarget.Thing.def.Fillage == FillCategory.Full)
			{
				projectileHitFlag |= ProjectileHitFlags.NonTargetWorld;
			}
			if (this.currentTarget.Thing != null)
			{
				ShootProjectile(projectileThing, launcher, launcher.DrawPos, this.currentTarget, this.currentTarget, projectileHitFlag, equipment, targetCoverDef);
			}
			else
			{
				ShootProjectile(projectileThing, launcher, launcher.DrawPos, shootLine.Dest, this.currentTarget, projectileHitFlag, equipment, targetCoverDef);
			}
			return true;
		}

		void ShootProjectile(Projectile proejctile, Thing launcher, Vector3 origin, LocalTargetInfo usedTarget, LocalTargetInfo intendedTarget, ProjectileHitFlags hitFlags, Thing equipment = null, ThingDef targetCoverDef = null)
		{
			/* 해당 위치 주변에 탄 뿌리기
			 * 개체와 거리 계산
			 * 거리에 따른 분산율 계산
			 * 분산율 비중에 따라 적중탄 비적중탄 구분
			 * 적중탄 발사
			 * 비적중탄 발사 위치 계산 (주변 랜덤 N개)
			 * 비적중탄 발사
			 */
			proejctile.Launch(launcher, origin, usedTarget, intendedTarget, hitFlags, equipment, targetCoverDef);
		}
	}
}